package de.hsbo.fbv.bmg.geometry.mysimple;

import java.awt.geom.Path2D;
import java.io.Serializable;

/**
 * Klasse SimpleCurve zur Speicherung linienfrmiger Objekte.
 * Konkrete Kindklasse von SimpleGeometry, die Linien im R oder R speichert.
 * @author A.Zimmermann, HS Bochum, 2011
 * @see BMG, Kap. 4.1
 * 
 */
public class SimpleCurve extends SimpleGeometry implements Serializable {

	/** Konstante: Versionsnummer */
	private static final long serialVersionUID = 1L;

	/** Attribut: SimplePoint-Array */
	SimplePoint[] points;

	/**
	 * Konstruktor aus SimplePoint-Array
	 * @param points
	 */
	public SimpleCurve(SimplePoint[] points) {
		this.points = points;
		this.setPath();
	}

	/**
	 * Gibt den Punkte mit dem Index i zurck. (Erster Punkt: i=0)
	 * @param i
	 * @return
	 */
	public SimplePoint get(int i) {
		return this.points[i];
	}

	/**
	 * Override-Methode von Object
	 */	
	public String toString() {
		String s = "";
		for (int i = 0; i < points.length; i++) {
			s = s + points[i].toString();
			if (i < points.length - 1)
				s = s + ", ";
		}
		return s;
	}

	/**
	 * Override-Methode von SimpleGeometry
	 */	
	public String toText() {
		return "CURVE(" + this.toString() + ")";
	}

	/**
	 * Gibt eine Kopie des Punkt-Arrays zurck.
	 * @return
	 */
	public SimplePoint[] getPoints() {
		SimplePoint[] pcopy = new SimplePoint[points.length];
		System.arraycopy(points, 0, pcopy, 0, pcopy.length);
		return pcopy;
	}

	/**
	 * Zerlegt einen Linenzug in seine Segmente. Die Endpunkte werden als
	 * Objekte neu generiert, um auch aneinander grenzende Liniensegmente in
	 * einer Map, deren Schlsselwerte die Endpunkte sind, verwalten zu knnen.
	 * 
	 * @return
	 */
	public SimpleLine[] getSegments() {
		SimpleLine[] segments = new SimpleLine[this.points.length - 1];
		for (int i = 0; i < this.points.length - 1; i++)
			segments[i] = new SimpleLine(new SimplePoint(points[i]),
					new SimplePoint(points[i + 1]));
		return segments;
	}

	/**
	 * Override-Methode von SimpleGeometry:<br>
	 * Erzeugt ein Graphik-Objekt zur Darstellung in einem Anzeigefenster.
	 * Es entsteht ein einfacher Streckenzug. Fr einen geschlossenen Streckenzug 
	 * mssen der Anfangs- und der Endpunkt lagegleich sein.
	 */
	public void setPath() {
		this.path = new Path2D.Double();
		this.path.moveTo(points[0].get(0), points[0].get(1));
		for (int i = 1; i < points.length; i++) {
			this.path.lineTo(points[i].get(0), points[i].get(1));
		}
	}

	/**
	 * Override-Methode von SimpleGeometry:<br>
	 * Gibt fr Linien die Anzahl der gespeicherten Sttzpunkte zurck.
	 */
	public int size() {
		return this.points.length;
	}

}
