package de.hsbo.fbv.bmg.geometry.mysimple;

import java.awt.geom.Path2D;

/**
 * Klasse SimpleSurface zur Speicherung flchenhafter Objekte.
 * Konkrete Kindklasse von SimpleGeometry, die Flchen im R oder R speichert.
 * @author A.Zimmermann, HS Bochum, 2011
 * @see BMG, Kap. 4.1
 * 
 */
public class SimpleSurface extends SimpleGeometry {
	
	/** Konstante: Versionsnummer */
	private static final long serialVersionUID = 1L;
	
	/** Attribut: SimpleCurve-Array */
	SimpleCurve[] curves;
	
	/**
	 * Konstruktor mit einem SimpleCurve-Objekt
	 * @param curve
	 */
	public SimpleSurface(SimpleCurve curve) {
		this.curves  = new SimpleCurve[] {curve};
		this.setPath();
	}

	/**
	 * Konstruktor mit mehreren SimpleCurve-Objekten
	 * @param points
	 */	
	public SimpleSurface(SimpleCurve[] curves) {
		this.curves  = curves;
		this.setPath();		
	}
	
	/**
	 * Gibt die Linie mit dem Index i zurck. (Erste Linie: i=0)
	 * @param i
	 * @return
	 */
	public SimpleCurve get(int i) {
		return this.curves[i];
	}
	
	/**
	 * Override-Methode von Object
	 */	
	public String toString() {
		String s = "";
		for(int i=0; i<curves.length; i++) {
			s = s + "("+curves[i].toString()+")";
			if(i<curves.length-1)
				s = s + " ";
		}
		return s;
	}
	
	/**
	 * Override-Methode von SimpleGeometry
	 */	
	public String toText() {
		return "SURFACE("+this.toString()+")";
	}
	
	/**
	 * Gibt eine Kopie des SimpleCurve-Arrays zurck.
	 * @return
	 */
	public SimpleCurve[] getCurves() {
		SimpleCurve[] ccopy = new SimpleCurve[curves.length];
		System.arraycopy(curves, 0, ccopy, 0, ccopy.length);
		return ccopy;
	}

	/**
	 * Gibt die Teilflchen zurck, die von den begrenzenden SimpleCurve-Elementen 
	 * gebildet werden. Fr jedes SimpleCurve-Objekt wird eine separate Flche 
	 * gebildet.
	 * @return
	 */
	public SimpleSurface[] getSurfaces() {
		SimpleSurface[] surfaces = new SimpleSurface[curves.length];
		for(int i=0; i<curves.length; i++)
			surfaces[i] = new SimpleSurface(curves[i]);
		return surfaces;
	}
		
	/**
	 * Override-Methode von SimpleGeometry:<br>
	 * Erzeugt ein Graphik-Objekt zur Darstellung in einem Anzeigefenster.
	 * Die komplexe Path2D-Instanz enthlt die Streckenzge der begrenzenden 
	 * SimpleCurve-Objekte.
	 */
	public void setPath() {
		this.path = new Path2D.Double();
		for(int j=0; j<curves.length; j++) {
			curves[j].path.closePath();
			this.path.append(curves[j].path, false);
		}
	}

	/**
	 * Override-Methode von SimpleGeometry:<br>
	 * Gibt fr Flchen die Anzahl der (geschlossenen) Begrenzungslinien an.
	 */
	public int size() {
		return this.curves.length;
	}

}
